@@ -16,21 +16,24 @@
 import os
 import sys
 import time
+import types
 
-from oslo_log import log as logging
-from oslo_service import service
-from oslo_utils import strutils
+import bunch
+import semver
 
 from muranoagent.common import config
 from muranoagent.common import messaging
+from muranoagent import exceptions as exc
 from muranoagent import execution_plan_queue
 from muranoagent import execution_plan_runner
 from muranoagent import execution_result as ex_result
-from muranoagent import validation
+from muranoagent.openstack.common import log as logging
+from muranoagent.openstack.common import service
 
 CONF = config.CONF
 
 LOG = logging.getLogger(__name__)
+format_version = '2.0.0'
 
 
 class MuranoAgent(service.Service):
@@ -44,7 +47,7 @@ class MuranoAgent(service.Service):
             LOG.debug('Loading plugin %s', name)
             __import__(name)
         except Exception:
-            LOG.warning('Cannot load package %s', name, exc_info=True)
+            LOG.warn('Cannot load package %s', name, exc_info=True)
             pass
 
     def _load(self):
@@ -74,54 +77,32 @@ class MuranoAgent(service.Service):
 
         plan = self._queue.get_execution_plan()
         if plan is not None:
-            LOG.debug("Got an execution plan '{0}':".format(
-                strutils.mask_password(str(plan))))
-            if self._verify_plan(plan):
-                self._run(plan)
+            LOG.debug("Got an execution plan '{0}':".format(str(plan)))
+            self._run(plan)
             return
 
-        next(msg_iterator)
-
-    def _verify_plan(self, plan):
-        try:
-            validation.validate_plan(plan)
-            return True
-        except Exception as err:
-            try:
-                execution_result = ex_result.ExecutionResult.from_error(
-                    err, plan)
-                if 'ReplyTo' in plan and CONF.enable_dynamic_result_queue:
-                    execution_result['ReplyTo'] = plan.ReplyTo
-
-                self._send_result(execution_result)
-            except ValueError:
-                LOG.warning('Execution result is not produced')
-            finally:
-                return False
+        msg_iterator.next()
 
     def _run(self, plan):
-        try:
-            with execution_plan_runner.ExecutionPlanRunner(plan) as runner:
+        with execution_plan_runner.ExecutionPlanRunner(plan) as runner:
+            try:
                 result = runner.run()
                 execution_result = ex_result.ExecutionResult.from_result(
                     result, plan)
                 self._queue.put_execution_result(execution_result, plan)
-        except Exception as ex:
-            LOG.exception('Error running execution plan')
-            execution_result = ex_result.ExecutionResult.from_error(ex,
-                                                                    plan)
-            self._queue.put_execution_result(execution_result, plan)
+            except Exception as ex:
+                LOG.exception('Error running execution plan')
+                execution_result = ex_result.ExecutionResult.from_error(ex,
+                                                                        plan)
+                self._queue.put_execution_result(execution_result, plan)
 
     def _send_result(self, result):
         with self._create_rmq_client() as mq:
             msg = messaging.Message()
             msg.body = result
             msg.id = result.get('SourceID')
-            routing_key = CONF.rabbitmq.result_routing_key
-            if ('ReplyTo' in result) and CONF.enable_dynamic_result_queue:
-                routing_key = result.pop('ReplyTo')
             mq.send(message=msg,
-                    key=routing_key,
+                    key=CONF.rabbitmq.result_routing_key,
                     exchange=CONF.rabbitmq.result_exchange)
         return True
 
@@ -134,9 +115,7 @@ class MuranoAgent(service.Service):
             'port': rabbitmq.port,
             'virtual_host': rabbitmq.virtual_host,
             'ssl': rabbitmq.ssl,
-            'ssl_version': rabbitmq.ssl_version,
-            'ca_certs': rabbitmq.ca_certs.strip() or None,
-            'insecure': rabbitmq.insecure
+            'ca_certs': rabbitmq.ca_certs.strip() or None
         }
         return messaging.MqClient(**connection_params)
 
@@ -149,22 +128,85 @@ class MuranoAgent(service.Service):
                                  prefetch_count=1) as subscription:
                         while True:
                             msg = subscription.get_message(timeout=5)
-                            if msg is not None:
-                                try:
-                                    self._queue.put_execution_plan(
-                                        msg.body,
-                                        msg.signature,
-                                        msg.id,
-                                        msg.reply_to)
-                                finally:
-                                    msg.ack()
+                            if msg is not None and isinstance(msg.body, dict):
+                                self._handle_message(msg)
 
-                            delay = 5
                             if msg is not None:
+                                msg.ack()
                                 yield
+                            delay = 5
             except KeyboardInterrupt:
                 break
             except Exception:
-                LOG.warning('Communication error', exc_info=True)
+                LOG.warn('Communication error', exc_info=True)
                 time.sleep(delay)
                 delay = min(delay * 1.2, 60)
+
+    def _handle_message(self, msg):
+        print(msg.body)
+        if 'ID' not in msg.body and msg.id:
+            msg.body['ID'] = msg.id
+        err = self._verify_plan(msg.body)
+        if err is None:
+            self._queue.put_execution_plan(msg.body)
+        else:
+            try:
+                execution_result = ex_result.ExecutionResult.from_error(
+                    err, bunch.Bunch(msg.body))
+
+                self._send_result(execution_result)
+            except ValueError:
+                LOG.warn('Execution result is not produced')
+
+    def _verify_plan(self, plan):
+        plan_format_version = plan.get('FormatVersion', '1.0.0')
+        if semver.compare(plan_format_version, '2.0.0') > 0 or \
+                semver.compare(plan_format_version, format_version) < 0:
+            range_str = 'in range 2.0.0-{0}'.format(plan_format_version) \
+                if format_version != '2.0.0' \
+                else 'equal to {0}'.format(format_version)
+            return exc.AgentException(
+                3,
+                'Unsupported format version {0} (must be {1})'.format(
+                    plan_format_version, range_str))
+
+        for attr in ('Scripts', 'Files', 'Options'):
+            if attr is plan and not isinstance(
+                    plan[attr], types.DictionaryType):
+                return exc.AgentException(
+                    2, '{0} is not a dictionary'.format(attr))
+
+        for name, script in plan.get('Scripts', {}).items():
+            for attr in ('Type', 'EntryPoint'):
+                if attr not in script or not isinstance(
+                        script[attr], types.StringTypes):
+                    return exc.AgentException(
+                        2, 'Incorrect {0} entry in script {1}'.format(
+                            attr, name))
+            if not isinstance(script.get('Options', {}), types.DictionaryType):
+                return exc.AgentException(
+                    2, 'Incorrect Options entry in script {0}'.format(name))
+
+            if script['EntryPoint'] not in plan.get('Files', {}):
+                return exc.AgentException(
+                    2, 'Script {0} misses entry point {1}'.format(
+                        name, script['EntryPoint']))
+
+            for additional_file in script.get('Files', []):
+                if additional_file not in plan.get('Files', {}):
+                    return exc.AgentException(
+                        2, 'Script {0} misses file {1}'.format(
+                            name, additional_file))
+
+        for key, plan_file in plan.get('Files', {}).items():
+            for attr in ('BodyType', 'Body', 'Name'):
+                if attr not in plan_file:
+                    return exc.AgentException(
+                        2, 'Incorrect {0} entry in file {1}'.format(
+                            attr, key))
+
+            if plan_file['BodyType'] not in ('Text', 'Base64'):
+                return exc.AgentException(
+                    2, 'Incorrect BodyType in file {1}'.format(key))
+
+        return None
