@@ -14,16 +14,15 @@
 # limitations under the License.
 
 import os
-import signal
 import stat
 import subprocess
 import sys
 
-from oslo_log import log as logging
+import bunch
 
-from muranoagent import bunch
 import muranoagent.exceptions
 from muranoagent import executors
+from muranoagent.openstack.common import log as logging
 
 LOG = logging.getLogger(__name__)
 
@@ -53,30 +52,23 @@ class ApplicationExecutor(object):
         script_name = os.path.relpath(self._path)
         LOG.debug("Starting '{0}' script execution".format(script_name))
 
-        popen_kwargs = {
-            'stdout': stdout,
-            'stderr': stderr,
-            'universal_newlines': True,
-            'cwd': dir_name,
-            'shell': True
-        }
-        if os.name != 'nt':
-            popen_kwargs['preexec_fn'] = lambda: signal.signal(
-                signal.SIGPIPE, signal.SIG_DFL)
-
-        process = subprocess.Popen(app, **popen_kwargs)
+        process = subprocess.Popen(
+            app,
+            stdout=stdout,
+            stderr=stderr,
+            universal_newlines=True,
+            cwd=dir_name,
+            shell=True)
         stdout, stderr = process.communicate(input)
         retcode = process.poll()
         LOG.debug("Script {0} execution finished "
                   "with retcode: {1}".format(script_name, retcode))
         if stdout is not None:
-            if hasattr(stdout, 'decode'):
-                stdout = stdout.decode('utf-8')
+            stdout = stdout.decode('utf-8')
             LOG.debug(u"'{0}' execution stdout: "
                       u"'{1}'".format(script_name, stdout))
         if stderr is not None:
-            if hasattr(stderr, 'decode'):
-                stderr = stderr.decode('utf-8')
+            stderr = stderr.decode('utf-8')
             LOG.debug(u"'{0}' execution stderr: "
                       u"'{1}'".format(script_name, stderr))
         result = {
